/*
 * modified for lecture by Fabian Fuchs (KIT, Karlsruhe) based on:
 *  
 Copyright (c) 2007, Distributed Computing Group (DCG)
                    ETH Zurich
                    Switzerland
                    dcg.ethz.ch

 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 - Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

 - Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the
   distribution.

 - Neither the name 'Sinalgo' nor the names of its contributors may be
   used to endorse or promote products derived from this software
   without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
package projects.randcolor;


import java.awt.Color;
import java.util.HashSet;

import javax.swing.JOptionPane;

import projects.randcolor.nodes.nodeImplementations.RandColorNode;
import sinalgo.configuration.Configuration;
import sinalgo.configuration.CorruptConfigurationEntryException;
import sinalgo.gui.helper.NodeSelectionHandler;
import sinalgo.nodes.Node;
import sinalgo.runtime.AbstractCustomGlobal;
import sinalgo.runtime.Global;
import sinalgo.runtime.Main;
import sinalgo.tools.Tools;
import sinalgo.tools.logging.Logging;

/**
 * This class holds customized global state and methods for the framework. 
 * The only mandatory method to overwrite is 
 * <code>hasTerminated</code>
 * <br>
 * Optional methods to override are
 * <ul>
 * <li><code>customPaint</code></li>
 * <li><code>handleEmptyEventQueue</code></li>
 * <li><code>onExit</code></li>
 * <li><code>preRun</code></li>
 * <li><code>preRound</code></li>
 * <li><code>postRound</code></li>
 * <li><code>checkProjectRequirements</code></li>
 * </ul>
 * @see sinalgo.runtime.AbstractCustomGlobal for more details.
 * <br>
 * In addition, this class also provides the possibility to extend the framework with
 * custom methods that can be called either through the menu or via a button that is
 * added to the GUI. 
 */
public class CustomGlobal extends AbstractCustomGlobal{
	
	private boolean noConflict;
	
	Logging log = Logging.getLogger("u6_log.txt");
	
	// The user can optionally specify exitAfter in the config file to indicate after how many rounds the simulation should stop. 
	boolean exitAfterFixedRounds = false;
	int exitAfterNumRounds;
	{
		if(Configuration.hasParameter("exitAfter")) {
			try {
				exitAfterFixedRounds = Configuration.getBooleanParameter("exitAfter");
			} catch (CorruptConfigurationEntryException e1) {
				Tools.fatalError("The 'exitAfter' needs to be a valid boolean.");
			}
			if(exitAfterFixedRounds) {
				try {
					exitAfterNumRounds = Configuration.getIntegerParameter("exitAfter/rounds");
				} catch (CorruptConfigurationEntryException e) {
					Tools.fatalError("The 'exitAfter/rounds' parameter specifies the maximum time the simulation runs. It needs to be a valid integer.");
				}
			}
		} else {
			exitAfterFixedRounds = false;
		}
	}
	
	
	/* (non-Javadoc)
	 * @see runtime.AbstractCustomGlobal#hasTerminated()
	 */
	public boolean hasTerminated(){
		if(exitAfterFixedRounds) {
			return exitAfterNumRounds <= Global.currentTime;
		}

		if(Tools.isSimulationInGuiMode()) {
			return false; // in GUI mode, have the user decide when to stop.
		} else {
			return noConflict || Global.currentTime > 100000; // stop after x rounds 
		}
	}

	/* can be used to experiment with maxDelta instead of numNeighbors */
	public int getMaxDelta() {
		int maxDelta = 0;
		int tmp = 0;
		for (Node v : Tools.getNodeList()) {
			maxDelta = (v.outgoingConnections.size() > maxDelta)?
					v.outgoingConnections.size():maxDelta;
		}
		return maxDelta;
	}
	
	/* (non-Javadoc)
	 * @see runtime.AbstractCustomGlobal#preRun()
	 */
	public void preRun() {
		noConflict = false;
	}
	
	public void preRound() {
		//do something before each round?
	}
	
	public void postRound() {
		noConflict = true;
		for (Node v: Tools.getNodeList()) {
			if (((RandColorNode)v).getColor() == Color.RED){
				noConflict = false;
			}
		}
		if (noConflict && Global.currentTime > 1) {
			log.logln("Round " + (int)(Global.currentTime) + " finished!");
			Tools.appendToOutput("Round " + (int)(Global.currentTime) + " finished!\n");
			
			//output to console despite: "outputToConsole=false" in batch 
			System.out.println("### " + (int)(Global.currentTime) + "\n");
			Tools.stopSimulation();
		}
		//double dt = System.currentTimeMillis() - Global.startTimeOfRound.getTime();
		//log.logln("Round " + (int)(Global.currentTime) + " time: "  + dt + " example text");
	}
			
	
	public void checkProjectRequirements(){
		//does the configuration fit the projects requirements?
		//Otherwise, call Main.fatalError("error message")
	}
	
	public void onExit() {
		// perform some cleanup operations here
	}
	
}
